Utilities
=========

This module contains various utility functions and classes used throughout the atomworks.ml package.

Debug Utilities
---------------

.. automodule:: atomworks.ml.utils.debug
   :members:
   :undoc-members:
   :show-inheritance:

Error Handling
--------------

.. automodule:: atomworks.ml.utils.error
   :members:
   :undoc-members:
   :show-inheritance:

Geometry Utilities
------------------

.. automodule:: atomworks.ml.utils.geometry
   :members:
   :undoc-members:
   :show-inheritance:

I/O Utilities
-------------

.. automodule:: atomworks.ml.utils.io
   :members:
   :undoc-members:
   :show-inheritance:

Miscellaneous Utilities
----------------------

.. automodule:: atomworks.ml.utils.misc
   :members:
   :undoc-members:
   :show-inheritance:

Nested Dictionary Utilities
--------------------------

.. automodule:: atomworks.ml.utils.nested_dict
   :members:
   :undoc-members:
   :show-inheritance:

NumPy Utilities
---------------

.. automodule:: atomworks.ml.utils.numpy
   :members:
   :undoc-members:
   :show-inheritance:

Random Number Generation
-----------------------

.. automodule:: atomworks.ml.utils.rng
   :members:
   :undoc-members:
   :show-inheritance:

Testing Utilities
-----------------

.. automodule:: atomworks.ml.utils.testing
   :members:
   :undoc-members:
   :show-inheritance:

Timer Utilities
---------------

.. automodule:: atomworks.ml.utils.timer
   :members:
   :undoc-members:
   :show-inheritance:

Token Utilities
---------------

.. automodule:: atomworks.ml.utils.token
   :members:
   :undoc-members:
   :show-inheritance: 